# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import flask
import unittest

from amundsen_application.models.user import load_user, dump_user, UserSchema

app = flask.Flask(__name__)
app.config.from_object('amundsen_application.config.LocalConfig')


def mock_get_profile_url(app: str) -> str:
    return 'testUrl'


app.config['GET_PROFILE_URL'] = mock_get_profile_url


class UserTest(unittest.TestCase):
    def test_set_user_id_from_email(self) -> None:
        """
        Deserialization and serialization sets user_id from email if no user_id
        :return:
        """
        with app.test_request_context():
            self.assertEqual(dump_user(load_user({'email': 'test@test.com'})).get('user_id'), 'test@test.com')

    def test_set_display_name_from_full_name(self) -> None:
        """
        Deserialization and serialization sets display_name from full_name if no display_name and
        full_name is a non-empty string
        :return:
        """
        test_user = {
            'email': 'test@test.com',
            'full_name': 'Test User',
        }
        with app.test_request_context():
            self.assertEqual(dump_user(load_user(test_user)).get('display_name'), 'Test User')

    def test_set_display_name_from_email(self) -> None:
        """
        Deserialization and serialization sets display_name from email if no display_name and
        full_name is None
        :return:
        """
        with app.test_request_context():
            self.assertEqual(dump_user(load_user({'email': 'test@test.com'})).get('display_name'), 'test@test.com')

    def test_set_display_name_from_email_if_full_name_empty(self) -> None:
        """
        Deserialization and serialization sets display_name from email if no display_name and
        full_name is ''
        :return:
        """
        test_user = {
            'email': 'test@test.com',
            'full_name': '',
        }
        with app.test_request_context():
            self.assertEqual(dump_user(load_user(test_user)).get('display_name'), 'test@test.com')

    def test_profile_url(self) -> None:
        """
        Deserialization and serialization sets profile_url from app.config['GET_PROFILE_URL']
        if no profile_url provided'
        :return:
        """
        with app.test_request_context():
            self.assertEqual(dump_user(load_user({'email': 'test@test.com'})).get('profile_url'), 'testUrl')

    def test_raise_error_if_no_display_name(self) -> None:
        """
        Error is raised if deserialization of Dict will not generate a display_name
        :return:
        """
        with app.test_request_context():
            data, errors = UserSchema().load({})
            self.assertEqual(len(errors['_schema']), 1)

    def test_raise_error_if_no_user_id(self) -> None:
        """
        Error is raised if deserialization of Dict will not generate a user_id
        :return:
        """
        with app.test_request_context():
            data, errors = UserSchema().load({'display_name': 'Test User'})
            self.assertEqual(len(errors['_schema']), 1)

    def test_str_no_value(self) -> None:
        """
        Test _str_no_value returns True for a string of spaces
        :return:
        """
        self.assertEqual(UserSchema()._str_no_value(' '), True)



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

from typing import Dict, Optional
from flask import Flask
from amundsen_application.config import LocalConfig
from amundsen_application.models.user import load_user, User


def get_access_headers(app: Flask) -> Optional[Dict]:
    """
    Function to retrieve and format the Authorization Headers
    that can be passed to various microservices who are expecting that.
    :param oidc: OIDC object having authorization information
    :return: A formatted dictionary containing access token
    as Authorization header.
    """
    try:
        access_token = app.oidc.get_access_token()
        return {'Authorization': 'Bearer {}'.format(access_token)}
    except Exception:
        return None


def get_auth_user(app: Flask) -> User:
    """
    Retrieves the user information from oidc token, and then makes
    a dictionary 'UserInfo' from the token information dictionary.
    We need to convert it to a class in order to use the information
    in the rest of the Amundsen application.
    :param app: The instance of the current app.
    :return: A class UserInfo (Note, there isn't a UserInfo class, so we use Any)
    """
    from flask import g
    user_info = load_user(g.oidc_id_token)
    return user_info


class OidcConfig(LocalConfig):
    AUTH_USER_METHOD = get_auth_user
    REQUEST_HEADERS_METHOD = get_access_headers



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import unittest
import json
from http import HTTPStatus

from flask import Response

from amundsen_application import create_app
from amundsen_application.api.preview import v0
from tests.unit.base.test_superset_preview_client import good_json_data, bad_json_data

local_app = create_app('amundsen_application.config.TestConfig', 'tests/templates')
PREVIEW_CLIENT_CLASS = 'amundsen_application.base.examples.example_superset_preview_client.SupersetPreviewClient'


class PreviewTest(unittest.TestCase):

    def setUp(self) -> None:
        local_app.config['PREVIEW_CLIENT_ENABLED'] = True

    def test_no_client_class(self) -> None:
        """
        Test that Not Implemented error is raised when PREVIEW_CLIENT is None
        :return:
        """
        # Reset side effects of other tests to ensure that the results are the
        # same regardless of execution order
        v0.PREVIEW_CLIENT_CLASS = None
        v0.PREVIEW_CLIENT_INSTANCE = None

        local_app.config['PREVIEW_CLIENT'] = None
        with local_app.test_client() as test:
            response = test.post('/api/preview/v0/')
            self.assertEqual(response.status_code, HTTPStatus.NOT_IMPLEMENTED)

    @unittest.mock.patch(PREVIEW_CLIENT_CLASS + '.get_preview_data')
    def test_good_client_response(self, mock_get_preview_data) -> None:
        """
        """
        expected_response_json = {
            'msg': 'Success',
            'previewData': {
                'columns': [{}, {}],
                'data': [{'id': 1, 'name': 'Admin'}, {'id': 2, 'name': 'Public'}, {'id': 3, 'name': 'Alpha'}]}
        }

        local_app.config['PREVIEW_CLIENT'] = PREVIEW_CLIENT_CLASS
        response = json.dumps({'preview_data': good_json_data})
        mock_get_preview_data.return_value = Response(response=response,
                                                      status=HTTPStatus.OK)
        with local_app.test_client() as test:
            response = test.post('/api/preview/v0/')
            self.assertEqual(response.status_code, HTTPStatus.OK)
            self.assertEqual(response.json, expected_response_json)

    @unittest.mock.patch(PREVIEW_CLIENT_CLASS + '.get_preview_data')
    def test_bad_client_response(self, mock_get_preview_data) -> None:
        """
        """
        expected_response_json = {
            'msg': 'Encountered exception: The preview client did not return a valid PreviewData object',
            'previewData': {}
        }

        local_app.config['PREVIEW_CLIENT'] = PREVIEW_CLIENT_CLASS
        response = json.dumps({'preview_data': bad_json_data})
        mock_get_preview_data.return_value = Response(response=response,
                                                      status=HTTPStatus.OK)
        with local_app.test_client() as test:
            response = test.post('/api/preview/v0/')
            self.assertEqual(response.status_code,
                             HTTPStatus.INTERNAL_SERVER_ERROR)
            self.assertEqual(response.json, expected_response_json)



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import logging
import os
import smtplib

from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from http import HTTPStatus
from typing import Dict, List

from flask import Response, jsonify, make_response

from amundsen_application.base.base_mail_client import BaseMailClient


#  based on https://stackoverflow.com/a/6270987
class MailClient(BaseMailClient):
    def __init__(self, recipients: List[str]) -> None:
        self.recipients = recipients

    def send_email(self,
                   html: str,
                   subject: str,
                   optional_data: Dict = None,
                   recipients: List[str] = None,
                   sender: str = None) -> Response:
        if not sender:
            sender = os.environ.get('AMUNDSEN_EMAIL') or ''  # set me
        if not recipients:
            recipients = self.recipients

        sender_pass = os.environ.get('AMUNDSEN_EMAIL_PASSWORD') or ''  # set me

        # Create message container - the correct MIME type
        # to combine text and html is multipart/alternative.
        msg = MIMEMultipart('alternative')
        msg['Subject'] = subject
        msg['From'] = sender
        msg['To'] = ', '.join(recipients)

        # Record the MIME type of text/html
        # and attach parts to message container.
        msg.attach(MIMEText(html, 'html'))

        s = smtplib.SMTP('smtp.gmail.com')
        try:
            s.connect('smtp.gmail.com', 587)
            s.ehlo()
            s.starttls()
            s.ehlo()
            s.login(sender, sender_pass)
            message = s.send_message(msg)
            payload = jsonify({'msg': message})
            s.quit()
            return make_response(payload, HTTPStatus.OK)
        except Exception as e:
            err_message = 'Encountered exception: ' + str(e)
            logging.exception(err_message)
            payload = jsonify({'msg': err_message})
            s.quit()
            return make_response(payload, HTTPStatus.INTERNAL_SERVER_ERROR)



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import logging

from typing import Dict  # noqa: F401

from flask import Response, jsonify, make_response


def create_error_response(*, message: str, payload: Dict, status_code: int) -> Response:
    """
    Logs and info level log with the given message, and returns a response with:
    1. The given message as 'msg' in the response data
    2. The given status code as thge response status code
    """
    logging.info(message)
    payload['msg'] = message
    return make_response(jsonify(payload), status_code)



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import json
import unittest
from http import HTTPStatus
from amundsen_application import create_app
from amundsen_application.proxy.issue_tracker_clients.issue_exceptions import IssueConfigurationException
from amundsen_application.models.data_issue import DataIssue, Priority
from amundsen_application.models.issue_results import IssueResults

local_app = create_app('amundsen_application.config.TestConfig', 'tests/templates')


class IssueTest(unittest.TestCase):

    def setUp(self) -> None:
        local_app.config['ISSUE_TRACKER_URL'] = 'url'
        local_app.config['ISSUE_TRACKER_CLIENT_ENABLED'] = True
        self.mock_issue = {
            'issue_key': 'key',
            'title': 'some title',
            'url': 'http://somewhere',
            'priority_name': 'Major',
            'priority_display_name': 'P2'
        }
        self.mock_issues = {
            'issues': [self.mock_issue]
        }
        self.mock_data_issue = DataIssue(issue_key='key',
                                         title='title',
                                         url='http://somewhere',
                                             status='open',
                                             priority=Priority.P2)
        self.expected_issues = IssueResults(issues=[self.mock_data_issue],
                                            total=0,
                                            all_issues_url="http://moredata")

    # ----- Jira API Tests ---- #

    def test_get_issues_not_enabled(self) -> None:
        """
        Test request sends ACCEPTED if not enabled
        :return:
        """
        local_app.config['ISSUE_TRACKER_CLIENT_ENABLED'] = False
        with local_app.test_client() as test:
            response = test.get('/api/issue/issues', query_string=dict(key='table_key'))
            self.assertEqual(response.status_code, HTTPStatus.ACCEPTED)

    @unittest.mock.patch('amundsen_application.api.issue.issue.get_issue_tracker_client')
    def test_get_jira_issues_missing_config(self, mock_issue_tracker_client) -> None:
        """
        Test request failure if config settings are missing
        :return:
        """
        local_app.config['ISSUE_TRACKER_URL'] = None
        mock_issue_tracker_client.return_value.get_issues.side_effect = IssueConfigurationException
        with local_app.test_client() as test:
            response = test.get('/api/issue/issues', query_string=dict(key='table_key'))
            self.assertEqual(response.status_code, HTTPStatus.NOT_IMPLEMENTED)

    def test_get_jira_issues_no_key(self) -> None:
        """
        Test request failure if table key is missing
        :return:
        """
        with local_app.test_client() as test:
            response = test.get('/api/issue/issues', query_string=dict(some_key='value'))
            self.assertEqual(response.status_code, HTTPStatus.INTERNAL_SERVER_ERROR)

    @unittest.mock.patch('amundsen_application.api.issue.issue.get_issue_tracker_client')
    def test_get_jira_issues_success(self, mock_issue_tracker_client) -> None:
        """
        Tests successful get request
        :return:
        """
        mock_issue_tracker_client.return_value.get_issues.return_value = self.expected_issues

        with local_app.test_client() as test:
            response = test.get('/api/issue/issues', query_string=dict(key='table_key'))
            data = json.loads(response.data)
            self.assertEqual(response.status_code, HTTPStatus.OK)
            self.assertEqual(data['issues']['issues'][0]['issue_key'],
                             self.expected_issues.issues[0].issue_key)
            self.assertEqual(data['issues']['total'],
                             self.expected_issues.total)
            self.assertEqual(data['issues']['all_issues_url'],
                             self.expected_issues.all_issues_url)
            mock_issue_tracker_client.return_value.get_issues.assert_called_with('table_key')

    def test_create_issue_not_enabled(self) -> None:
        """
        Test request sends ACCEPTED if not enabled
        :return:
        """
        local_app.config['ISSUE_TRACKER_CLIENT_ENABLED'] = False
        with local_app.test_client() as test:
            response = test.post('/api/issue/issue', data={
                'description': 'test description',
                'title': 'test title',
                'key': 'key'
            })
            self.assertEqual(response.status_code, HTTPStatus.ACCEPTED)

    @unittest.mock.patch('amundsen_application.api.issue.issue.get_issue_tracker_client')
    def test_create_jira_issue_missing_config(self, mock_issue_tracker_client) -> None:
        """
        Test request failure if config settings are missing
        :return:
        """
        mock_issue_tracker_client.side_effect = IssueConfigurationException
        local_app.config['ISSUE_TRACKER_URL'] = None
        with local_app.test_client() as test:
            response = test.post('/api/issue/issue', data={
                'description': 'test description',
                'title': 'test title',
                'key': 'key'
            })
            self.assertEqual(response.status_code, HTTPStatus.NOT_IMPLEMENTED)

    def test_create_jira_issue_no_description(self) -> None:
        """
         Test request failure if table key is missing
         :return:
         """
        with local_app.test_client() as test:
            response = test.post('/api/issue/issue', data={
                'key': 'table_key',
                'title': 'test title',
            })
            self.assertEqual(response.status_code, HTTPStatus.INTERNAL_SERVER_ERROR)

    def test_create_jira_issue_no_key(self) -> None:
        """
         Test request failure if table key is missing
         :return:
         """
        with local_app.test_client() as test:
            response = test.post('/api/issue/issue', data={
                'description': 'test description',
                'title': 'test title'
            })
            self.assertEqual(response.status_code, HTTPStatus.INTERNAL_SERVER_ERROR)

    def test_create_jira_issue_no_title(self) -> None:
        """
         Test request failure if table key is missing
         :return:
         """
        with local_app.test_client() as test:
            response = test.post('/api/issue/issue', data={
                'description': 'test description',
                'key': 'table_key',
            })
            self.assertEqual(response.status_code, HTTPStatus.INTERNAL_SERVER_ERROR)

    @unittest.mock.patch('amundsen_application.api.issue.issue.get_issue_tracker_client')
    def test_create_jira_issue_success(self, mock_issue_tracker_client) -> None:
        """
        Test request returns success and expected outcome
        :return:
        """
        mock_issue_tracker_client.return_value.create_issue.return_value = self.mock_data_issue

        with local_app.test_client() as test:
            response = test.post('/api/issue/issue',
                                 content_type='multipart/form-data',
                                 data={
                                     'description': 'test description',
                                     'title': 'title',
                                     'key': 'key'
                                 })
            data = json.loads(response.data)
            self.assertEqual(response.status_code, HTTPStatus.OK)
            mock_issue_tracker_client.assert_called
            mock_issue_tracker_client.return_value.create_issue.assert_called
            self.assertEqual(data['issue'].get('title'), 'title')
            self.assertEqual(data['issue'].get('issue_key'), 'key')



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import logging
from abc import ABCMeta, abstractmethod

from amundsen_application.base.base_preview import BasePreview
from amundsen_application.api.preview.dashboard.dashboard_preview.mode_preview import ModePreview

LOGGER = logging.getLogger(__name__)


class BasePreviewMethodFactory(metaclass=ABCMeta):

    @abstractmethod
    def get_instance(self, *, uri: str) -> BasePreview:
        """
        Provides an instance of BasePreview based on uri
        :param uri:
        :return:
        """
        pass


class DefaultPreviewMethodFactory(BasePreviewMethodFactory):

    def __init__(self) -> None:
        # Register preview clients here. Key: product, Value: BasePreview implementation
        self._object_map = {
            'mode': ModePreview()
        }
        LOGGER.info('Supported products: {}'.format(list(self._object_map.keys())))

    def get_instance(self, *, uri: str) -> BasePreview:
        product = self.get_product(uri=uri)

        if product in self._object_map:
            return self._object_map[product]

        raise NotImplementedError('Product {} is not supported'.format(product))

    def get_product(self, *, uri: str) -> str:
        return uri.split('_')[0]



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

# flake8: noqa
import unittest

from flask import current_app

from amundsen_application import create_app
from amundsen_application.api.preview.dashboard.dashboard_preview.mode_preview import ModePreview
from amundsen_application.api.preview.dashboard.dashboard_preview.preview_factory_method import DefaultPreviewMethodFactory


class TestDefaultPreviewFactory(unittest.TestCase):

    def setUp(self) -> None:
        self.app = create_app(config_module_class='amundsen_application.config.LocalConfig')
        self.app_context = self.app.app_context()
        self.app_context.push()
        self.app.config['CREDENTIALS_MODE_ADMIN_TOKEN'] = 'CREDENTIALS_MODE_ADMIN_TOKEN'
        self.app.config['CREDENTIALS_MODE_ADMIN_PASSWORD'] = 'CREDENTIALS_MODE_ADMIN_PASSWORD'
        self.app.config['MODE_ORGANIZATION'] = 'foo'

    def tearDown(self) -> None:
        self.app_context.pop()

    def test_app_exists(self) -> None:
        self.assertFalse(current_app is None)

    def test_factory(self) -> None:
        factory = DefaultPreviewMethodFactory()
        actual = factory.get_instance(uri='mode_dashboard://gold.dg_id/d_id')

        self.assertTrue(isinstance(actual, ModePreview))

        try:
            factory.get_instance(uri='tableau_dashboard://foo.bar/baz')
            self.assertTrue(False, 'Should have failed for we currently do not support Tableau')
        except Exception:
            pass



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import asana
import logging
from typing import Dict, List

from amundsen_application.base.base_issue_tracker_client import BaseIssueTrackerClient
from amundsen_application.models.data_issue import DataIssue, Priority
from amundsen_application.models.issue_results import IssueResults


class AsanaClient(BaseIssueTrackerClient):

    def __init__(self, issue_labels: List[str],
                 issue_tracker_url: str,
                 issue_tracker_user: str,
                 issue_tracker_password: str,
                 issue_tracker_project_id: int,
                 issue_tracker_max_results: int) -> None:
        self.issue_labels = issue_labels
        self.asana_url = issue_tracker_url
        self.asana_user = issue_tracker_user
        self.asana_password = issue_tracker_password
        self.asana_max_results = issue_tracker_max_results

        self.asana_project_gid = issue_tracker_project_id
        self.asana_client = asana.Client.access_token(issue_tracker_password)

        asana_project = self.asana_client.projects.get_project(self.asana_project_gid)
        self.asana_workspace_gid = asana_project['workspace']['gid']

        self._setup_custom_fields()

    def get_issues(self, table_uri: str) -> IssueResults:
        """
        :param table_uri: Table Uri ie databasetype://database/table
        :return: Metadata of matching issues
        """

        table_parent_task_gid = self._get_parent_task_gid_for_table_uri(table_uri)

        tasks = list(self.asana_client.tasks.get_subtasks_for_task(
            table_parent_task_gid,
            {
                'opt_fields': [
                    'name', 'completed', 'notes', 'custom_fields',
                ]
            }
        ))

        return IssueResults(
            issues=[
                self._asana_task_to_amundsen_data_issue(task) for task in tasks
            ],
            total=len(tasks),
            all_issues_url=self._task_url(table_parent_task_gid),
        )

    def create_issue(self, table_uri: str, title: str, description: str) -> DataIssue:
        """
        Creates an issue in Jira
        :param description: Description of the Jira issue
        :param table_uri: Table Uri ie databasetype://database/table
        :param title: Title of the Jira ticket
        :return: Metadata about the newly created issue
        """

        table_parent_task_gid = self._get_parent_task_gid_for_table_uri(table_uri)

        return self._asana_task_to_amundsen_data_issue(
            self.asana_client.tasks.create_subtask_for_task(
                table_parent_task_gid,
                {
                    'name': title,
                    'notes': description,
                }
            )
        )

    def _setup_custom_fields(self) -> None:
        TABLE_URI_FIELD_NAME = 'Table URI (Amundsen)'
        PRIORITY_FIELD_NAME = 'Priority (Amundsen)'

        custom_fields = \
            self.asana_client.custom_field_settings.get_custom_field_settings_for_project(
                self.asana_project_gid
            )

        custom_fields = {f['custom_field']['name']: f['custom_field'] for f in custom_fields}

        if TABLE_URI_FIELD_NAME in custom_fields:
            table_uri_field = custom_fields[TABLE_URI_FIELD_NAME]
        else:
            table_uri_field = self.asana_client.custom_fields.create_custom_field({
                'workspace': self.asana_workspace_gid,
                'name': TABLE_URI_FIELD_NAME,
                'format': 'custom',
                'resource_subtype': 'text',
            })

            self.asana_client.projects.add_custom_field_setting_for_project(
                self.asana_project_gid,
                {
                    'custom_field': table_uri_field['gid'],
                    'is_important': True,
                }
            )

        if PRIORITY_FIELD_NAME in custom_fields:
            priority_field = custom_fields[PRIORITY_FIELD_NAME]
        else:
            priority_field = self.asana_client.custom_fields.create_custom_field({
                'workspace': self.asana_workspace_gid,
                'name': PRIORITY_FIELD_NAME,
                'format': 'custom',
                'resource_subtype': 'enum',
                'enum_options': [
                    {
                        'name': p.level
                    } for p in Priority
                ]
            })

            self.asana_client.projects.add_custom_field_setting_for_project(
                self.asana_project_gid,
                {
                    'custom_field': priority_field['gid'],
                    'is_important': True,
                }
            )

        self.table_uri_field_gid = table_uri_field['gid']
        self.priority_field_gid = priority_field['gid']

    def _get_parent_task_gid_for_table_uri(self, table_uri: str) -> str:
        table_parent_tasks = list(self.asana_client.tasks.search_tasks_for_workspace(
            self.asana_workspace_gid,
            {
                'projects.any': [self.asana_project_gid],
                'custom_fields.{}.value'.format(self.table_uri_field_gid): table_uri,
            }
        ))

        # Create the parent task if it doesn't exist.
        if len(table_parent_tasks) == 0:
            table_parent_task = self.asana_client.tasks.create_task({
                'name': table_uri,
                'custom_fields': {
                    self.table_uri_field_gid: table_uri,
                },
                'projects': [self.asana_project_gid],
            })

            return table_parent_task['gid']
        else:
            if len(table_parent_tasks) > 1:
                logging.warn('There are currently two tasks with the name "{}"'.format(table_uri))

            return table_parent_tasks[0]['gid']

    def _task_url(self, task_gid: str) -> str:
        return 'https://app.asana.com/0/{project_gid}/{task_gid}'.format(
            project_gid=self.asana_project_gid, task_gid=task_gid
        )

    def _asana_task_to_amundsen_data_issue(self, task: Dict) -> DataIssue:
        custom_fields = {f['gid']: f for f in task['custom_fields']}
        priority_field = custom_fields[self.priority_field_gid]

        priority = None
        if priority_field.get('enum_value'):
            priority = Priority.from_level(priority_field['enum_value']['name'])
        else:
            priority = Priority.P3

        return DataIssue(
            issue_key=task['gid'],
            title=task['name'],
            url=self._task_url(task['gid']),
            status='closed' if task['completed'] else 'open',
            priority=priority,
        )



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import json
import unittest

from amundsen_application import create_app
from amundsen_application.api.utils.response_utils import create_error_response

local_app = create_app('amundsen_application.config.TestConfig', 'tests/templates')


class ResponseUtilsTest(unittest.TestCase):
    def setUp(self) -> None:
        pass

    def test_create_error_response(self) -> None:
        """
        Verify that the returned response contains the given messag and status_code
        :return:
        """
        test_message = 'Success'
        test_payload = {}
        status_code = 200
        with local_app.app_context():
            response = create_error_response(message=test_message,
                                             payload=test_payload,
                                             status_code=status_code)
        data = json.loads(response.data)
        self.assertEqual(response.status_code, status_code)
        self.assertEqual(data.get('msg'), test_message)



# Copyright Contributors to the Amundsen project.
# SPDX-License-Identifier: Apache-2.0

import flask
import json
import unittest

from http import HTTPStatus
from requests import Response
from typing import Dict
from unittest.mock import Mock

from amundsen_application.base.base_superset_preview_client import BaseSupersetPreviewClient

app = flask.Flask(__name__)
app.config.from_object('amundsen_application.config.LocalConfig')

good_json_data = {
    "columns": [
        {
            "agg": "count_distinct",
            "is_date": False,
            "type": "INT",
            "name": "id",
            "is_dim": False,
        },
        {
            "is_date": False,
            "type": "STRING",
            "name": "name",
            "is_dim": True,
        }
    ],
    "data": [
        {"id": 1, "name": "Admin"},
        {"id": 2, "name": "Public"},
        {"id": 3, "name": "Alpha"},
    ]
}
expected_results = {
    "columns": [
        {
            "column_type": "INT",
            "column_name": "id",
        },
        {
            "column_type": "STRING",
            "column_name": "name",
        }
    ],
    "data": [
        {"id": 1, "name": "Admin"},
        {"id": 2, "name": "Public"},
        {"id": 3, "name": "Alpha"},
    ],
    "error_text": ""
}

bad_json_data = {
    "columns": [
        {
            "agg": "count_distinct",
            "is_date": False,
            "type": "INT",
            "name": "id",
            "is_dim": False,
        },
        {
            "is_date": False,
            "type": "STRING",
            "name": "name",
            "is_dim": True,
        }
    ],
    "data": "Wrong type",
}


class MockClient(BaseSupersetPreviewClient):
    def __init__(self) -> None:
        super().__init__()

    def post_to_sql_json(self, *, params: Dict, headers: Dict) -> Response:
        mockresponse = Mock()
        mockresponse.json.return_value = good_json_data
        mockresponse.status_code = HTTPStatus.OK
        return mockresponse


class MockBadDataClient(BaseSupersetPreviewClient):
    def __init__(self) -> None:
        self.headers = {}

    def post_to_sql_json(self, *, params: Dict, headers: Dict) -> Response:
        mockresponse = Mock()
        mockresponse.json.return_value = bad_json_data
        mockresponse.status_code = HTTPStatus.OK
        return mockresponse


class MockExceptionClient(BaseSupersetPreviewClient):
    def __init__(self) -> None:
        super().__init__()

    def post_to_sql_json(self, *, params: Dict, headers: Dict) -> Response:
        mockresponse = Mock()
        mockresponse.json.return_value = None
        mockresponse.status_code = HTTPStatus.OK
        return mockresponse


class SupersetPreviewClientTest(unittest.TestCase):
    def test_get_preview_data_raise_exception(self) -> None:
        """
        Test catching any exception raised in get_preview_data(), which should result in
        a response with 500 error and empty preview_data payload
        :return:
        """
        with app.test_request_context():
            response = MockExceptionClient().get_preview_data(params={})
            self.assertEqual(json.loads(response.data).get('preview_data'), {})
            self.assertEqual(response.status_code, HTTPStatus.INTERNAL_SERVER_ERROR)

    def test_post_sql_json_incorrect_data_shape(self) -> None:
        """
        Test catching errors in the data shape returned by post_sql_json(), which should result in
        a response with 500 error and empty preview_data payload
        :return:
        """
        with app.test_request_context():
            response = MockBadDataClient().get_preview_data(params={})
            self.assertEqual(json.loads(response.data).get('preview_data'), {})
            self.assertEqual(response.status_code, HTTPStatus.INTERNAL_SERVER_ERROR)

    def test_post_sql_json_correct_data_shape(self) -> None:
        """
        Test post_sql_json(), which should result in
        a response with 500 error and empty preview_data payload
        :return:
        """
        with app.test_request_context():
            response = MockClient().get_preview_data(params={}, optionalHeaders={'testKey': 'testValue'})
            self.assertEqual(json.loads(response.data).get('preview_data'), expected_results)
            self.assertEqual(response.status_code, HTTPStatus.OK)
